/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Help CSS.
 *
 * The Initial Developer of the Original Code is
 * R.J. Keller.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

@import url("chrome://global/skin/");
@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

toolbar toolbarbutton {
	-moz-box-orient: horizontal !important;
	min-width: 16px;
	list-style-image: url("chrome://help/skin/Toolbar.png");
}

toolbarbutton:not([disabled="true"]):hover,
menubutton:not([disabled="true"]):hover,
toolbarbutton:not([disabled="true"]):hover:active,
menubutton:not([disabled="true"]):hover:active {
	color: ButtonText !important;
}

/* Set the minimum sidebar width so the help contents aren't squeezed together.*/
#help-sidebar { min-width: 15em; width: 20em; max-width: 25em; }

/** *** BEGIN TOOLBAR BUTTON CODE ***
 * All toolbar button images are 30x30 for the Help Viewer.
 */
 #help-back-button { -moz-image-region: rect(0px 30px 30px 0px); }
 #help-back-button:hover { -moz-image-region: rect(30px 30px 60px 0px); }
 #help-back-button[disabled="true"] { -moz-image-region: rect(60px 30px 90px 0px); }

 #help-forward-button { -moz-image-region: rect(0px 60px 30px 30px); }
 #help-forward-button:hover { -moz-image-region: rect(30px 60px 60px 30px); }
 #help-forward-button[disabled="true"] { -moz-image-region: rect(60px 60px 90px 30px); }

 #help-home-button { -moz-image-region: rect(0px 90px 30px 60px); }
 #help-home-button:hover { -moz-image-region: rect(30px 90px 60px 60px); }

 #help-print-button { -moz-image-region: rect(0px 120px 30px 90px); }
 #help-print-button:hover { -moz-image-region: rect(30px 120px 60px 90px); }
