//Microsoft Developer Studio generated resource script.
//
#include "resrc1.h"

#define APSTUDIO_READONLY_SYMBOLS
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 2 resource.
//
#include "resource.h"

/////////////////////////////////////////////////////////////////////////////
#undef APSTUDIO_READONLY_SYMBOLS

/////////////////////////////////////////////////////////////////////////////
// English (U.S.) resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_ENU)
#ifdef _WIN32
LANGUAGE LANG_ENGLISH, SUBLANG_ENGLISH_US
#pragma code_page(1252)
#endif //_WIN32

/////////////////////////////////////////////////////////////////////////////
//
// Icon
//

// Icon with lowest ID value placed first to ensure application icon
// remains consistent on all systems.
IDI_MAIN                ICON    DISCARDABLE     "preloader.ico"
IDI_CHECK               ICON    DISCARDABLE     "check.ico"
IDI_CHECK1              ICON    DISCARDABLE     "check1.ico"
IDI_BROWSERRUNNING      ICON    DISCARDABLE     "icon1.ico"

/////////////////////////////////////////////////////////////////////////////
//
// Dialog
//

DISABLE DIALOG DISCARDABLE  0, 0, 186, 63
STYLE DS_MODALFRAME | DS_NOFAILCREATE | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "Disable Preloader?"
FONT 8, "MS Sans Serif"
BEGIN
    DEFPUSHBUTTON   "OK",IDOK,129,7,50,14
    PUSHBUTTON      "Cancel",IDCANCEL,129,24,50,14
    LTEXT           "Are you sure you want to disable the preloader, after all of our hard work?",
                    IDC_STATIC,20,13,89,33
END

IDD_PERFORMANCE DIALOG DISCARDABLE  0, 0, 233, 95
STYLE DS_MODALFRAME | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "Performance Configuration"
FONT 8, "MS Sans Serif"
BEGIN
    DEFPUSHBUTTON   "OK",IDOK,168,42,50,14
    CONTROL         "Slider1",IDC_MODULES,"msctls_trackbar32",TBS_AUTOTICKS | 
                    TBS_BOTH | WS_TABSTOP,57,17,100,19
    CONTROL         "Slider2",IDC_ENTRIES,"msctls_trackbar32",TBS_AUTOTICKS | 
                    TBS_BOTH | WS_TABSTOP,57,37,100,19
    CONTROL         "Slider3",IDC_FREQUENCY,"msctls_trackbar32",
                    TBS_AUTOTICKS | TBS_BOTH | WS_TABSTOP,57,58,100,19
    LTEXT           "Modules",IDC_STATIC,17,26,28,8
    LTEXT           "Entry Points",IDC_STATIC,17,45,38,8
    LTEXT           "Frequency",IDC_STATIC,17,66,34,8
    LTEXT           "Min",IDC_STATIC,61,8,12,8
    LTEXT           "Max",IDC_STATIC,143,9,14,8
    PUSHBUTTON      "Cancel",IDCANCEL,168,60,50,14
END


/////////////////////////////////////////////////////////////////////////////
//
// Menu
//

IDR_MENU1 MENU DISCARDABLE 
BEGIN
    POPUP "Popup"
    BEGIN
        MENUITEM "&Disable",                    IDM_REMOVE
        MENUITEM "&Configure",                  IDM_CONFIGURE
        MENUITEM "E&xit",                       IDM_EXIT
    END
END


#ifdef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// TEXTINCLUDE
//

1 TEXTINCLUDE DISCARDABLE 
BEGIN
    "resrc1.h\0"
END

2 TEXTINCLUDE DISCARDABLE 
BEGIN
    "#include ""resource.h""\r\n"
    "\0"
END

3 TEXTINCLUDE DISCARDABLE 
BEGIN
    "\r\n"
    "\0"
END

#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// DESIGNINFO
//

#ifdef APSTUDIO_INVOKED
GUIDELINES DESIGNINFO DISCARDABLE 
BEGIN
    DISABLE, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 179
        TOPMARGIN, 7
        BOTTOMMARGIN, 56
    END

    IDD_PERFORMANCE, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 226
        TOPMARGIN, 7
        BOTTOMMARGIN, 88
    END
END
#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// String Table
//

STRINGTABLE DISCARDABLE 
BEGIN
    IDS_MOZ_KEY             "Software\\Mozilla.org\\Mozilla\\0.8 (en)\\Main"
    IDS_NS65_KEY            "Software\\Netscape\\Netscape 6\\6.5 (en)\\Main"
    IDS_NS60_KEY            "Software\\Netscape\\Netscape 6\\6.0 (en)\\Main"
    IDS_PRE60_KEY           "Software\\MicrosoftWindows\\CurrentVersion\\App Paths\netscp6.exe"
    IDS_STARTUP_FOLDER_PREFIX "Shortcut to "
    IDS_STARTUP_FOLDER_SUFFIX ".lnk"
    IDS_STARTUP_FOLDER_KEY  "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders"
    IDS_STARTUP_FOLDER_SUBKEY1 "Startup"
    IDS_STARTUP_FOLDER_SUBKEY2 "Common Startup"
    IDS_CLASSNAME           "MozPreLoader"
    IDS_TOOLTIP             "Right click to configure the preloader"
    IDS_SUBKEY_INSTALL      "Install Directory"
    IDS_SUBKEY_PATH         "Path"
    IDS_NS_WINDOWNAME       "- Netscape 6"
    IDS_MOZ_WINDOWNAME      "- Mozilla"
END

#endif    // English (U.S.) resources
/////////////////////////////////////////////////////////////////////////////



#ifndef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 3 resource.
//


/////////////////////////////////////////////////////////////////////////////
#endif    // not APSTUDIO_INVOKED

